Rem
* Example 001 HelloWorld
This Tutorial shows how To set up the IDE For using the
Irrlicht Engine And how To write a simple HelloWorld
program with it. The program will show how To use the
basics of the VideoDriver, the GUIEnvironment And the
SceneManager.

To use the engine, we will have To Import the BlitzMax module 
Irrlicht.Core, which needs to be manually installed into your 
BlitzMax mod folder from the distribution.  Please see the 
distribution for information on installation.  If you are using 
Linux or MacOSX you may need to recompile the module the first time.

That's it, with your IDE set up like this, you will now be able to
develop applications with the Irrlicht Engine.
EndRem

SuperStrict
Framework Irrlicht.Core

Rem
The most important Function of the engine is the createDevice()
Function. The IrrlichtDevice is created by it, which is the root
Object For doing anything with the engine. createDevice() has 7
parameters:

- deviceType: Type of the device. This can currently be the Null-device,
   one of the two software renderers, D3D8, D3D9, Or OpenGL. In this
   example we use EDT_SOFTWARE, but To Try out, you might want To
   change it To EDT_BURNINGSVIDEO, EDT_NULL, EDT_DIRECT3D8,
   EDT_DIRECT3D9, Or EDT_OPENGL.

- windowSize: Size of the Window Or screen in FullScreenMode To be
   created. In this example we use 640x480.

- bits: Amount of color bits per pixel. This should be 16 Or 32. The
   parameter is often ignored when running in windowed mode.

- fullscreen: Specifies If we want the device To run in fullscreen mode
   Or Not.

- stencilbuffer: Specifies If we want To use the stencil buffer (For
   drawing shadows).

- vsync: Specifies If we want To have vsync enabled, this is only useful
   in fullscreen mode.

- eventReceiver: An Object To receive events. We do Not want To use this
   parameter here, And set it To 0.

Always check the Return value To cope with unsupported drivers,
dimensions, etc.
EndRem

?MacOS
Local device:IrrlichtDevice = ..
	IrrlichtDevice.Create(EDT_OPENGL, _DIMENSION2DI(640, 480), 16, ..
		False, False, False, Null)
?Not MacOS
Local device:IrrlichtDevice = ..
	IrrlichtDevice.Create(EDT_BURNINGVIDEO, _DIMENSION2DI(640, 480), 16, ..
		False, False, False, Null)
?

If Not device Then Return
		
Rem
Set the caption of the window To some nice text.
EndRem
device.setWindowCaption("Hello World! - Irrlicht Engine Demo")

Rem
Get a pointer To the VideoDriver, the SceneManager And the graphical
user interface environment, so that we do Not always have To write
device->getVideoDriver(), device->getSceneManager(), Or
device->getGUIEnvironment().
EndRem
Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()
Local guienv:IGUIEnvironment = device.getGUIEnvironment()

Rem
We add a hello world label To the window, using the GUI environment.
The text is placed at the position (10,10) as top Left corner And
(260,22) as Lower Right corner.
EndRem
guienv.addStaticText("Hello World! This is the Irrlicht Software renderer!", ..
	_RECTI(10,10,260,22), True)
	
Rem
To show something interesting, we Load a Quake 2 model And display it.
We only have To get the Mesh from the Scene Manager with getMesh() And add
a SceneNode To display the mesh with addAnimatedMeshSceneNode(). We
check the Return value of getMesh() To become aware of loading problems
And other errors.

Instead of writing the filename sydney.md2, it would also be possible
To Load a Maya Object file (.obj), a complete Quake3 map (.bsp) Or any
other supported file format. By the way, that cool Quake 2 model
called sydney was modelled by Brian Collins.
EndRem
Local mesh:IAnimatedMesh = smgr.getMesh("../../media/sydney.md2")
If Not mesh Then Return
Local node:IAnimatedMeshSceneNode = smgr.addAnimatedMeshSceneNode( mesh )

Rem
To let the mesh look a little bit nicer, we change its material. We
disable lighting because we do Not have a dynamic light in here, And
the mesh would be totally black otherwise. Then we set the frame loop,
such that the predefined STAND animation is used. And last, we apply a
texture To the mesh. Without it the mesh would be drawn using only a
color.
EndRem
If node
	node.setMaterialFlag(EMF_LIGHTING, False)
	node.setMD2Animation ( EMAT_STAND )
	node.setMaterialTexture( 0, driver.getTexture("../../media/sydney.bmp") )
EndIf

Rem
To look at the mesh, we place a camera into 3d space at the position
(0, 30, -40). The camera looks from there To (0,5,0), which is
approximately the place where our md2 model is.
EndRem
smgr.addCameraSceneNode(Null, _VECTOR3DF(0,30,-40), _VECTOR3DF(0,5,0))

Rem
Ok, now we have set up the scene, lets draw everything: We run the
device in a While() loop, Until the device does Not want To run any
more. This would be when the user closes the window Or presses ALT+F4
(Or whatever keycode closes a window).
EndRem
While device.run() 

	Rem
	Anything can be drawn between a beginScene() And an endScene()
	call. The beginScene() call clears the screen with a color And
	the depth buffer, If desired. Then we let the Scene Manager And
	the GUI Environment draw their content. With the endScene()
	call everything is presented on the screen.
	EndRem
	driver.beginScene(True, True, _SCOLOR(255,100,101,140))

	smgr.drawAll()
	guienv.drawAll()

	driver.endScene()
EndWhile

Rem
After we are done with the render loop, we have To Delete the Irrlicht
Device created before with createDevice(). In the Irrlicht Engine, you
have To Delete all objects you created with a Method Or Function which
starts with 'create'. The object is simply deleted by calling ->drop().
See the documentation at irr::IReferenceCounted::drop() For more
information.
EndRem
device.drop()

Rem
That's it. Compile and run.
EndRem
