Rem
* Example 003 Custom SceneNode

This Tutorial is more advanced than the previous ones.
If you are currently just playing around with the Irrlicht
engine, you may want To look at other examples first.
This tutorials shows how To Create a custom scene node And
how To use it in the engine. A custom scene node is needed
If you want To implement a render technique the Irrlicht
Engine currently does Not support. For example, you can write
an indoor portal based renderer Or an advanced terrain scene
node with it. By creating custom scene nodes, you can
easily extend the Irrlicht Engine And adapt it To your own
needs.

I will keep the tutorial simple: Keep everything very
Short, everything in one .cpp file, And I'll use the engine
here as in all other tutorials.

To start, I Include the header files, use the irr namespace,
And tell the linker To link with the .lib file.
EndRem
SuperStrict
Framework Irrlicht.Core

Rem
Here comes the most sophisticated part of this tutorial: 
The class of our very own custom scene node. To keep it simple,
our scene node will Not be an indoor portal renderer nor a terrain
scene node, but a simple tetraeder, a 3d Object consiting of 4
connected vertices, which only draws itself And does nothing more.
Note that this scenario does Not require a custom scene node in Irrlicht.
Instead one would Create a mesh from the geometry And pass it To an
IMeshSceneNode. This example just illustrates creation of a custom
scene node in a very simple setting.

To let our scene node be able To be inserted into the Irrlicht 
Engine scene, the class we create needs only be derived from the
ISceneNode class And has To override some methods.
EndRem

Type CSampleSceneNode Extends ISceneNode

	Rem
	First, we declare some member variables:
	The bounding box, 4 vertices, And the material of the tetraeder.
	EndRem
	Field Box:AABbox3df
	Field Vertices:Array_S3DVertex
	Field Material:SMaterial

	Rem
	The parameters of the constructor specify the parent of the scene node,
	a pointer To the scene manager, And an id of the scene node.
	In the constructor we call the parent class' constructor,
	set some properties of the material, And
	Create the 4 vertices of the tetraeder we will draw later.
	EndRem
	Method New()
		Box=AABbox3df.Create()
		Material=SMaterial.Create()
		Vertices=Array_S3DVertex.createWithSize(4)	
		Material.setWireframe(False)
		Material.setLighting(False)
		
		Vertices.insert(S3DVertexDefault.createFromVals(0,0,10,1,1,0,_SCOLOR(255,0,255,255), 0, 1),0)
		Vertices.insert(S3DVertexDefault.createFromVals(10,0,-10, 1,0,0, _SCOLOR(255,255,0,255), 1, 1),1)
		Vertices.insert(S3DVertexDefault.createFromVals(0,20,0, 0,1,1, _SCOLOR(255,255,255,0), 1, 0),2)
		Vertices.insert(S3DVertexDefault.createFromVals(-10,0,-10, 0,0,1, _SCOLOR(255,0,255,0), 0, 0),3)

		Rem
		The Irrlicht Engine needs To know the bounding box of a scene node.
		It will use it For automatic culling And other things. Hence, we
		need To Create a bounding box from the 4 vertices we use.
		If you do Not want the engine To use the box For automatic culling,
		And/Or don't want to create the box, you could also call
		irr::scene::ISceneNode::setAutomaticCulling() with irr::scene::EAC_OFF.
		EndRem
		Box.resetFromVect(Vertices.elementAt(0).getPos())
		For Local i:Int=1 To 3
			Box.addInternalPoint(Vertices.elementAt(i).getPos())
		Next
	EndMethod

	Rem
	Before it is drawn, the irr::scene::ISceneNode::OnRegisterSceneNode()
	Method of every scene node in the scene is called by the scene manager.
	If the scene node wishes To draw itself, it may register itself in the
	scene manager To be drawn. This is necessary To tell the scene manager
	when it should call irr::scene::ISceneNode::render(). For
	example, normal scene nodes render their content one after another,
	While stencil buffer shadows would like To be drawn after all other
	scene nodes. And camera Or light scene nodes need To be rendered before
	all other scene nodes (If at all). So here we simply register the
	scene node To render normally. If we would like To let it be rendered
	like cameras Or light, we would have To call
	SceneManager->registerNodeForRendering(this, SNRT_LIGHT_AND_CAMERA);
	After this, we call the actual
	irr::scene::ISceneNode::OnRegisterSceneNode() Method of the base class,
	which simply lets also all the child scene nodes of this node register
	themselves.
	EndRem
	Method OnRegisterSceneNode()
		If (isVisible())
			getSceneManager().registerNodeForRendering(Self)
		EndIf

		Super.OnRegisterSceneNode()
	EndMethod
	
	Rem
	In the render() Method most of the interesting stuff happens: The
	Scene node renders itself. We override this Method And draw the
	tetraeder.
	EndRem
	Method render()
		' Irrlicht.Core use Irrlicht arrays for all array interaction
		Local indicies:Array_u16=Array_u16.createWithSize(12)
		indicies.insert(0)
		indicies.insert(2)
		indicies.insert(3)
		
		indicies.insert(2)
		indicies.insert(1)
		indicies.insert(3)

		indicies.insert(1)
		indicies.insert(0)
		indicies.insert(3)
		
		indicies.insert(2)
		indicies.insert(0)
		indicies.insert(1)

		Local driver:IVideoDriver = getSceneManager().getVideoDriver()

		driver.setMaterial(Material)
		driver.SetTransform(ETS_WORLD,getAbsoluteTransformation())
		driver.drawIndexedTriangleList(Vertices,indicies)
	EndMethod
	
	Rem
	And finally we Create three small additional methods.
	irr::scene::ISceneNode::getBoundingBox() returns the bounding box of
	this scene node, irr::scene::ISceneNode::getMaterialCount() returns the
	amount of materials in this scene node (our tetraeder only has one
	material), And irr::scene::ISceneNode::getMaterial() returns the
	material at an index. Because we have only one material here, we can
	Return the only one material, assuming that no one ever calls
	getMaterial() with an index greater than 0.
	EndRem
	Method getBoundingBox:AABbox3df()
		Return Box
	EndMethod

	Method getMaterialCount:Int()
		Return 1
	EndMethod

	Method getMaterial:SMaterial(i:Int)
		Return Material
	EndMethod	
	
	' we must override the generate function in order for instantiation to work properly
	Function generate:ISceneNode()
		Return New CSampleSceneNode
	EndFunction
EndType

Rem
That's it. The Scene node is done. Now we simply have to start
the engine, Create the scene node And a camera, And look at the result.
EndRem
' let user Select driver Type

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' Create device

Local device:IrrlichtDevice= ..
	IrrlichtDevice.Create(EDT_OPENGL, _DIMENSION2DI(640, 480), 16, False)
	
If Not device Return ' could Not create selected driver.

' Create engine And camera

device.setWindowCaption("Custom Scene Node - Irrlicht Engine Demo")

Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()

smgr.addCameraSceneNode(Null, _VECTOR3DF(0,-40,0), _VECTOR3DF(0,0,0))

Rem
Create our scene node. I don't check the result of calling new, as it
should Throw an exception rather than returning 0 on failure. Because
the New node will Create itself with a reference count of 1, And Then
will have another reference added by its parent scene node when it is
added To the scene, I need To drop my reference To it. Best practice is
To drop it only *after* I have finished using it, regardless of what
the reference count of the Object is after creation.
EndRem
Local myNode:CSampleSceneNode = CSampleSceneNode(CSampleSceneNode.Create( ..
	CSampleSceneNode.generate,smgr.getRootSceneNode(), smgr, 666))

Rem
To Animate something in this boring scene consisting only of one
tetraeder, And To show that you now can use your scene node like any
other scene node in the engine, we add an animator To the scene node,
which rotates the node a little bit.
irr::scene::ISceneManager::createRotationAnimator() could Return 0, so
should be checked.
EndRem
Local anim:ISceneNodeAnimator = ..
	smgr.createRotationAnimator(_VECTOR3DF(0.8, 0, 0.8))

If anim
	myNode.addAnimator(anim)
	
	Rem
	I'm done referring to anim, so must
	IReferenceCounted.drop() this reference now because it
	was produced by a createFoo() Function. As I shouldn't refer to
	it again, ensure that I can't by setting to 0.
	EndRem
	anim.drop()
	anim = Null ' As I shouldn't refer to it again, ensure that I can't
EndIf

Rem
I'm done with my CSampleSceneNode object, and so must drop my reference.
This won't delete the object, yet, because it is still attached to the
scene graph, which prevents the deletion Until the graph is deleted Or the
custom scene node is removed from it.
EndRem
myNode.drop()
myNode = Null ' As I shouldn't refer to it again, ensure that I can't

Rem
Now draw everything And finish.
EndRem
Local frames:Int=0
While(device.run())
	
	driver.beginScene(True, True, _SCOLOR(0,100,100,100))

	smgr.drawAll()

	driver.endScene()
	frames:+1
	
	If (frames=100)
		Local str:String = "Irrlicht Engine ["
		str :+ driver.getName()
		str :+ "] FPS: "
		str :+ driver.getFPS()

		device.setWindowCaption(str)
		frames=0
	EndIf
Wend

device.drop()

Rem
That's it. Compile and play around with the program.
EndRem
